# -*- coding: utf-8 -*-
from plone.app.linkintegrity.upgrades import migrate_linkintegrity_relations
from plone.app.linkintegrity.utils import hasIncomingLinks
from plone.app.linkintegrity.utils import referencedRelationship
try:
    from Products.Archetypes.interfaces import IReferenceable
    from plone.app.linkintegrity.tests.base import ATBaseTestCase
    HAS_AT = True
except ImportError:
    HAS_AT = False
import unittest


if HAS_AT:
    class ReferenceMigrationATTestCase(ATBaseTestCase):
        """Reference migration testcase for at content types"""

        @unittest.skipUnless(
            HAS_AT, 'Archetypes are not installed. Skipping migration tests')
        def test_upgrade(self):
            doc3 = self.portal['doc3']
            doc1 = self.portal['doc1']
            self.assertTrue(IReferenceable.providedBy(doc3))
            doc3.setText('<a href="doc1">doc1</a>', mimetype='text/html')
            doc3.addReference(doc1, relationship=referencedRelationship)
            self.assertFalse(hasIncomingLinks(doc1))
            self.assertFalse(hasIncomingLinks(doc3))
            migrate_linkintegrity_relations(self.portal)
            self.assertTrue(hasIncomingLinks(doc1))
            self.assertFalse(hasIncomingLinks(doc3))

else:

    class TestDummy(unittest.TestCase):

        def test_dummy(self):
            pass



# -*- coding: utf-8 -*-
# If Archetypes is not installed, define dummy objects
# to replace Archetypes imports.
try:
    from Products.Archetypes.interfaces import IBaseObject
    from Products.Archetypes.Field import TextField
except ImportError:
    from zope.interface import Interface

    class IBaseObject(Interface):
        pass

    class TextField(object):
        pass



# -*- coding: utf-8 -*-
from plone.app.linkintegrity import testing
from plone.app.relationfield.behavior import IRelatedItems
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.textfield import RichTextValue
from plone.testing.z2 import Browser
from z3c.form.interfaces import IFormLayer
from zope.component import getMultiAdapter
from zope.interface import alsoProvides
from zope.lifecycleevent import modified

import six
import unittest


class BaseTestCase(unittest.TestCase):

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        alsoProvides(self.request, IFormLayer)

        # Get a testbrowser
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False
        self.browser.addHeader('Referer', self.portal.absolute_url())
        self.browser.addHeader(
            'Authorization',
            'Basic {0:s}:{1:s}'.format(TEST_USER_NAME, TEST_USER_PASSWORD))

        # Do an initial page load to make sure the bundles get compiled
        # (which currently commits a transaction)
        # before we render exception views
        self.browser.open(self.portal.absolute_url())

        setRoles(self.portal, TEST_USER_ID, ['Manager', ])

    def _get_token(self, obj):
        return getMultiAdapter(
            (obj, self.request), name='authenticator').token()


class DXBaseTestCase(BaseTestCase):
    """Base testcase for testing Dexterity content types"""

    layer = testing.PLONE_APP_LINKINTEGRITY_DX_FUNCTIONAL_TESTING

    def _set_text(self, obj, text):
        obj.text = RichTextValue(text)
        modified(obj)

    def _get_text(self, obj):
        return obj.text.raw

    def _set_related_items(self, obj, items):
        assert IRelatedItems.providedBy(obj)
        setattr(obj, 'relatedItems', items)
        modified(obj)

    def _get_related_items(self, obj):
        return obj.relatedItems


if six.PY2:
    class ATBaseTestCase(BaseTestCase):
        """Base testcase for testing Archetypes content types"""

        layer = testing.PLONE_APP_LINKINTEGRITY_AT_FUNCTIONAL_TESTING

        def _set_text(self, obj, text):
            obj.setText(text, mimetype='text/html')
            modified(obj)

        def _get_text(self, obj):
            # This is the equivalent to obj.text in dexterity. No transforms,
            # no rewritten relative urls
            return obj.getText(raw=1).raw

        def _set_related_items(self, obj, items):
            obj.setRelatedItems(items)
            modified(obj)

        def _get_related_items(self, obj):
            return obj.getRelatedItems()



# -*- coding: utf-8 -*-
from base64 import decodestring
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_MIGRATION_FIXTURE  # noqa
from plone.app.testing import layers
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import ploneSite
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.namedfile.file import NamedImage
from plone.testing import z2
from Products.CMFCore.utils import getToolByName
from zope.configuration import xmlconfig

import six


B64_DATA = b'R0lGODlhAQABAIAAAAUEBAAAACwAAAAAAQABAAACAkQBADs='
GIF = six.BytesIO(decodestring(B64_DATA))
GIF.filename = 'sample.gif'
GIF.contentType = 'image/gif'
GIF._width = 1
GIF._height = 1


def create(container, type_name, **kwargs):
    """A easy helper method to create some content since we do not have
    plone.api in core.
    """

    new_id = container.invokeFactory(type_name, **kwargs)
    content = container[new_id]

    # Archetypes specific code was taken from ``plone.api``
    # Switch when api has been merged into core.
    if six.PY2:
        from Products.Archetypes.interfaces import IBaseObject
        if IBaseObject.providedBy(content):
            content.processForm()

    return content


class LinkIntegrityLayer(z2.Layer):
    """Base Layer for AT and Dexterity testing.
    """

    defaultBases = (PLONE_FIXTURE, )

    def setUpMembers(self, portal):
        pm = getToolByName(portal, 'portal_membership')
        pm.addMember('editor', TEST_USER_PASSWORD, ['Editor'], [])
        pm.addMember('member', TEST_USER_PASSWORD, ['Member'], [])
        pm.addMember('authenticated', TEST_USER_PASSWORD, [], [])

    def setUpContent(self):
        import plone.app.linkintegrity
        xmlconfig.file('configure.zcml', plone.app.linkintegrity,
                       context=self['configurationContext'])

        with z2.zopeApp() as app:
            z2.installProduct(app, 'plone.app.linkintegrity')

        with ploneSite() as portal:
            setRoles(portal, TEST_USER_ID, ['Manager', ])
            login(portal, TEST_USER_NAME)

            # Create sample documents
            type_data = dict(type_name='Document')
            for i in range(1, 4):
                type_data['id'] = 'doc{0:d}'.format(i)
                type_data['title'] = 'Test Page {0:d}'.format(i)
                create(portal, **type_data)

            create(portal, 'File', id='file1', title='File 1', file=GIF)
            create(portal, 'Folder', id='folder1', title='Folder 1')
            subfolder = portal['folder1']
            create(subfolder, 'Document', id='doc4', title='Test Page 4')

            self.setUpMembers(portal)

    def tearDown(self):
        with z2.zopeApp() as app:
            z2.uninstallProduct(app, 'plone.app.linkintegrity')


PLONE_APP_LINKINTEGRITY_FIXTURE = LinkIntegrityLayer()


class LinkIntegrityATLayer(LinkIntegrityLayer):
    """Layer which targets testing with Archetypes and ATContentTypes.
    """

    directory = 'at'
    defaultBases = (
        PLONE_APP_CONTENTTYPES_MIGRATION_FIXTURE,
        PLONE_APP_LINKINTEGRITY_FIXTURE,
    )

    def setUp(self):
        self.setUpContent()

    def setUpContent(self):
        super(LinkIntegrityATLayer, self).setUpContent()

        with ploneSite() as portal:
            create(portal, 'Image', id='image1', title='Image 1', image=GIF)


PLONE_APP_LINKINTEGRITY_AT_FIXTURE = LinkIntegrityATLayer()


class LinkIntegrityDXLayer(LinkIntegrityLayer):
    """Layer which targets testing with Dexterity.
    """

    directory = 'dx'
    defaultBases = (
        PLONE_APP_CONTENTTYPES_FIXTURE,
        PLONE_APP_LINKINTEGRITY_FIXTURE,
    )

    def setUp(self):
        self.setUpContent()

    def setUpContent(self):
        super(LinkIntegrityDXLayer, self).setUpContent()

        with ploneSite() as portal:
            # Create an object that does not provide the behavior to live along
            create(portal, 'News Item', id='news1', title='News 1')

            # create a DX NamedImage
            portal.invokeFactory('Image', 'image1')
            portal['image1'].image = NamedImage(GIF, 'image/gif',
                                                u'sample.gif')


PLONE_APP_LINKINTEGRITY_DX_FIXTURE = LinkIntegrityDXLayer()

PLONE_APP_LINKINTEGRITY_DX_INTEGRATION_TESTING = layers.IntegrationTesting(
    bases=(PLONE_APP_LINKINTEGRITY_DX_FIXTURE, ),
    name='plone.app.linkintegrity:DX:Integration'
)

PLONE_APP_LINKINTEGRITY_DX_FUNCTIONAL_TESTING = layers.FunctionalTesting(
    bases=(PLONE_APP_LINKINTEGRITY_DX_FIXTURE, ),
    name='plone.app.linkintegrity:DX:Functional'
)

if six.PY2:
    PLONE_APP_LINKINTEGRITY_AT_INTEGRATION_TESTING = layers.IntegrationTesting(
        bases=(PLONE_APP_LINKINTEGRITY_AT_FIXTURE, ),
        name='plone.app.linkintegrity:AT:Integration'
    )

    PLONE_APP_LINKINTEGRITY_AT_FUNCTIONAL_TESTING = layers.FunctionalTesting(
        bases=(PLONE_APP_LINKINTEGRITY_AT_FIXTURE, ),
        name='plone.app.linkintegrity:AT:Functional'
    )



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


version = '3.3.8'

setup(
    name='plone.app.linkintegrity',
    version=version,
    description='Manage link integrity in Plone.',
    long_description='\n\n'.join([
        open("README.rst").read(),
        open('CHANGES.rst').read(),
    ]),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 5.1",
        "Framework :: Plone :: 5.2",
        "Framework :: Zope2",
        "Framework :: Zope :: 4",
        "Intended Audience :: Other Audience",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Internet :: WWW/HTTP :: Site Management :: Link Checking",
    ],
    keywords='link integrity plone',
    author='Plone Foundation',
    author_email='plone-developers@lists.sourceforge.net',
    url='https://pypi.org/project/plone.app.linkintegrity',
    license='GPL version 2',
    packages=find_packages(),
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    install_requires=[
        'setuptools',
        'six',
        'plone.app.intid',
        'plone.app.relationfield',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            'plone.app.contenttypes',
            'plone.app.dexterity [relations]',  # related items in dx 2.0
        ],
    },
    platforms='Any',
    zip_safe=False,
    entry_points='''
    [z3c.autoinclude.plugin]
    target = plone
    ''',
)



# -*- coding: utf-8 -*-
from six.moves.html_parser import HTMLParser

try:
    from html.parser import HTMLParseError
except ImportError as e:
    # HTMLParseError is removed in Python 3.5. Since it can never be
    # thrown in 3.5, we can just define our own class as a placeholder.
    class HTMLParseError(Exception):
        pass


class LinkParser(HTMLParser):
    """A simple html parser for link and image urls.
    """

    def __init__(self):
        HTMLParser.__init__(self)
        self.links = []

    def getLinks(self):
        """Return all links found during parsing.
        """
        return tuple(self.links)

    def handle_starttag(self, tag, attrs):
        """Override the method to remember all links.
        """
        if tag == 'a':
            self.links.extend(search_attr('href', attrs))
        if tag == 'img':
            self.links.extend(search_attr('src', attrs))


def search_attr(name, attrs):
    """Search named attribute in a list of attributes.
    """
    for attr, value in attrs:
        if attr == name:
            return [value]
    return []


def extractLinks(data, encoding='utf-8'):
    """Parse the given html and return all links.
    """
    if not data:
        return []
    parser = LinkParser()
    try:
        parser.feed(data)
        parser.close()
    except UnicodeDecodeError:
        parser = LinkParser()
        parser.feed(data.decode(encoding))
        parser.close()
    except (HTMLParseError, TypeError):
        pass

    return parser.getLinks()
